/* 
    kirc -- KDE Infrared Remote Control

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef kirc__h
#define kirc__h

#include <qdict.h>
#include <qlistview.h>
#include <kconfig.h>
#include <kircdata.h>
#include <qsocketnotifier.h> 
#include <qlineedit.h> 

// comment out the next line if you want to use lirc 0.4.1
#define LIRC_VERSION_036

// =================================================================================================
// =================================================================================================

class KIRCConfig : public QWidget
{

  Q_OBJECT

public:
  KIRCConfig(QDict<KCodeEntry> *_aCodeDict, QWidget * parent = 0, const char * name = 0 );
  ~KIRCConfig();


public slots:
  void receiveCodeSLOT( ullong code );
  void newDictSLOT( QDict<KCodeEntry> *_aCodeDict );

private slots:
  void applySLOT();
  void defaultsSLOT();
  void cancelSLOT();

  void selectionChangedSLOT( QListViewItem * item );
  void textChangedSLOT( const char *);

signals:
  void apply();
  void defaults();
  void cancel();

private:
  QListView *lv;
  QDict<KCodeEntry> *aCodeDict;

  QLineEdit *le;

  QListViewItem *currentItem;
  KCodeEntry *currentCode;

  bool newDictInsert;
};

// =================================================================================================
// =================================================================================================

class KIRCWidget : public QObject
{

  Q_OBJECT

public:
  KIRCWidget( KConfig* pconf, unsigned short int _port= 12345 );
  ~KIRCWidget();

  bool addCode( const QString& functionName, uint defaultCode, 
		KCodeEntry::codeType type = KCodeEntry::normal,
		bool configurable = TRUE );

  void removeCode( const QString& functionName );
  
  unsigned int readCurrentKey( const QString& functionName );
  unsigned int readDefaultKey( const QString& functionName );

  void registerWidget( const QString& widgetName, QWidget* currentWidget );
  
  void connectFunction( const QString& widgetName,const QString& functionName,
			const QObject* receiver, const char* member,
			bool activate = TRUE );

  void disconnectFunction( const QString& widgetName,const QString& functionName );
  void toggleFunction( const QString& widgetName,const QString& functionName, bool activate );
  void disconnectAllFunctions( const QString& widgetName );

  void destroyWidgetEntry( const QString& widgetName );

  void sync();

  bool configureCodes( QWidget *parent );

signals:
  void numKeyReceivedSIGNAL( int dec );
  void specialKeyReceivedSIGNAL( KCodeEntry::codeType );

  // internal use only
  void configDataReceivedSIGNAL( ullong data );

public slots:
  void reconnectToSocket( unsigned short int _port );
  void applySLOT();
  void defaultsSLOT();
  void cancelSLOT();

private slots:
  void dataReceivedSLOT( int socket );
	
private:
  bool open_socket_to_irc( unsigned short int _port );
  bool close_socket_to_irc();
  /*
  void internalDisconnectFunction( const QString& widgetName, 
				   KIRCWidgetEntry *pWEntry, KCodeEntry *pEntry, KCodeConnectEntry *pCEntry );
				   */
  void internalDisconnectAll( const QString& widgetName);

  KConfig *pConfig;
  
  KIRCConfig *kc;

  int aAvailableId;
  QDict<KCodeEntry> aCodeDict;
  QDict<KIRCWidgetEntry> aWidgetDict;

  QSocketNotifier *sn;
  unsigned short int port;
  int sockfd;                                 // socket identifier

  KCodeEntry::codeType keyUP;
  int num,num10;

  unsigned char buf[8];
  ullong data;
  
  bool configMode;
};




#endif
